from google.cloud import secretmanager
from dask.distributed import Client
import waterhealer as wh
import json
import os
from typing import List
from . import streaming

__version__ = '0.0.14'
client = secretmanager.SecretManagerServiceClient()

try:
    secret = os.environ.get('kafka_bootstrap')
    payload = json.loads(
        client.access_secret_version(secret).payload.data.decode('utf-8')
    )
    kafka_bootstrap = payload.get('bootstrap')
except:
    raise Exception('please export a correct variable `kafka_bootstrap`.')

try:
    secret = os.environ.get('dask_bootstrap')
    payload = json.loads(
        client.access_secret_version(secret).payload.data.decode('utf-8')
    )
    dask_bootstrap = payload.get('bootstrap')

except:
    raise Exception('please export a correct variable `dask_bootstrap`.')


def get_source(
    topics: List[str],
    consumer: str,
    get_dask: bool = True,
    boilerplates: List[str] = None,
    **kwargs,
):
    """
    Get Kafka source and Dask client.

    Parameters
    ----------
    topics : List[str]
        list of strings of topics want to subscribe.
    consumer : str
        consumer name.
    get_dask : bool, optional (default=True)
        connect to dask cluster.
    boilerplates : List[str], optional (default=None)
        list of strings of filename for import, example `['boilerplate_fave_payments.py']`

    Returns
    -------
    result: (kafka_source, dask_client)
    """

    source = wh.from_kafka(
        topics,
        {
            'bootstrap.servers': kafka_bootstrap,
            'group.id': consumer,
            'auto.offset.reset': kwargs.get('offset', 'earliest'),
            'max.poll.interval.ms': kwargs.get('maxpoll', 30_000_000),
        },
        maxage_memory = kwargs.get('maxage', 9_999_999),
    )
    if get_dask:

        def reload_package(package):
            import sys
            import importlib

            try:

                importlib.reload(sys.modules[package])
            except:
                pass

            return list(sys.modules.keys())

        client = Client(dask_bootstrap, loop = source.loop)
        if boilerplates:
            for boilerplate in boilerplates:
                client.upload_file(boilerplate)
                client.run(reload_package, boilerplate.replace('.py', ''))
    else:
        client = None
    return source, client
